@@ -183,7 +183,21 @@ extern "C" {
     QPDF_ERROR_CODE qpdf_read(qpdf_data qpdf, char const* filename,
 			      char const* password);
 
-    /* Read functions below must be called after qpdf_read. */
+    /* Calling qpdf_read_memory causes processMemoryFile to be called
+     * in the C++ API.  Otherwise, it behaves in the same way as
+     * qpdf_read.  The description argument will be used in place of
+     * the file name in any error or warning messages generated by the
+     * library.
+     */
+    QPDF_DLL
+    QPDF_ERROR_CODE qpdf_read_memory(qpdf_data qpdf,
+				     char const* description,
+				     char const* buffer,
+				     unsigned long size,
+				     char const* password);
+
+    /* Read functions below must be called after qpdf_read or
+     * qpdf_read_memory. */
 
     /* Return the version of the PDF file. */
     QPDF_DLL
